<strong>
  {% if type == "substitution" and item.start_period == item.end_period %}
    {{ el.lesson_period.period.period }}.
  {% elif type == "substitution"  %}
    {{ item.start_period }}.–{{ item.end_period }}.
  {% elif type == "extra_lesson" %}
    {{ el.period.period }}.
  {% elif type == "event" %}
    {% if el.period_from_on_day == el.period_to_on_day %}
      {{ el.period_from_on_day }}.
    {% else %}
      {{ el.period_from_on_day }}.–{{ el.period_to_on_day }}.
    {% endif %}
  {% elif type == "supervision_substitution" %}
    {% with break=el.supervision.break_item %}
      {{ break.after_period_number }}./{{ break.before_period_number }}.
    {% endwith %}
  {% endif %}
</strong>
