{% load i18n %}

{% if affected_teachers or affected_groups or absent_teachers or absent_groups %}
  <div class="{% if not print %}card{% endif %}">
    <div class="{% if not print %}card-content{% endif %}">
      {% if absent_teachers %}
        <div class="row no-margin">
          <div class="col {% if not print %}s12 m3{% else %}s3{% endif %}">
            <strong class="truncate">
              {% trans "Absent teachers" %}
            </strong>
          </div>
          <div class="col {% if not print %}s12 m9{% else %}s9{% endif %} black-text-a">
            {% include "chronos/partials/teachers.html" with teachers=absent_teachers %}
          </div>
        </div>
      {% endif %}
      {% if absent_groups %}
        <div class="row no-margin">
          <div class="col {% if not print %}s12 m3{% else %}s3{% endif %}">
            <strong class="truncate">
              {% trans "Absent groups" %}
            </strong>
          </div>
          <div class="col {% if not print %}s12 m9{% else %}s9{% endif %} black-text-a">
            {% include "chronos/partials/groups.html" with groups=absent_groups no_collapsible=True %}
          </div>
        </div>
      {% endif %}
      {% if affected_teachers %}
        <div class="row no-margin">
          <div class="col {% if not print %}s12 m3{% else %}s3{% endif %}">
            <strong class="truncate">
              {% trans "Affected teachers" %}
            </strong>
          </div>
          <div class="col {% if not print %}s12 m9{% else %}s9{% endif %} black-text-a">
            {% include "chronos/partials/teachers.html" with teachers=affected_teachers %}
          </div>
        </div>
      {% endif %}
      {% if affected_groups %}
        <div class="row no-margin">
          <div class="col {% if not print %}s12 m3{% else %}s3{% endif %}">
            <strong class="truncate">
              {% trans "Affected groups" %}
            </strong>
          </div>
          <div class="col {% if not print %}s12 m9{% else %}s9{% endif %} black-text-a">
            {% include "chronos/partials/groups.html" with groups=affected_groups no_collapsible=True %}
          </div>
        </div>
      {% endif %}
    </div>
  </div>
  {% if print %}<br/>{% endif %}
{% endif %}
