{# -*- engine:django -*- #}

{% extends "core/base.html" %}
{% load i18n %}


{% load render_table from django_tables2 %}

{% block browser_title %}{% blocktrans %}Lessons{% endblocktrans %}{% endblock %}
{% block no_page_title %}{% endblock %}

{% block content %}
  <script type="text/javascript">
    var dest = Urls.lessonsDay();
  </script>

  <div class="row no-margin">
    <div class="col s12 m6 l8 no-padding">
      <h1>{% blocktrans %}Lessons{% endblocktrans %} {{ day|date:"l" }}, {{ day }}</h1>
    </div>
    <div class="col s12 m6 l4 no-padding">
      {% include "chronos/partials/datepicker.html" %}
    </div>
  </div>

  {% render_table lessons_table %}
{% endblock %}
