{# -*- engine:django -*- #}

{% extends 'core/base.html' %}

{% load i18n static %}

{% block extra_head %}
  <link rel="stylesheet" href="{% static 'css/chronos/timetable.css' %}">
{% endblock %}

{% block browser_title %}{% blocktrans %}My timetable{% endblocktrans %}{% endblock %}
{% block no_page_title %}{% endblock %}

{% block content %}
  <div class="row no-margin">
    <div class="col s12">
      <h1>
        {% trans "My timetable" %} <i>{{ el }}</i>
        <span class="badge new primary-color ">{% trans "SMART PLAN" %}</span>
      </h1>
      <a class="btn-flat waves-effect waves-light" href="{% url "timetable" super.type.value super.el.pk %}">
        {% trans "Show week timetable for" %} {{ super.el.short_name }}
      </a>
    </div>
  </div>

  <div class="row nomargin hide-on-large-only">
    <div class="col m12 s12 l6 xl4">
      {% include "core/partials/announcements.html" with announcements=announcements %}
    </div>
  </div>

  <div class="row nomargin hide-on-large-med-and-down">
    <div class="col m12 s12 l6 xl4">
      {% include "core/partials/announcements.html" with announcements=week_announcements %}
    </div>
  </div>

    <div class="row">
    <div class="col s12">
      <div class="card timetable-title-card">
        <div class="card-content">
            <span class="card-title">
                {% include "chronos/partials/datepicker.html" with display_date_only=1 %}
                <span class="show-on-medium-and-down hide-on-large-only">
                {% if weekday.date == today %}
                    <br/> {% include "chronos/partials/today.html" %}
                {% endif %}
                </span>
            </span>
        </div>
      </div>

    </div>
  </div>

  <div class="row hide-on-large-only">
    <div class="timetable-plan col s12 m12 xl4">
      {#  Lessons #}
      {% include "chronos/partials/lessons_col.html" with lesson_periods=lesson_periods %}
    </div>
  </div>
  <div class="row timetable-plan hide-on-med-and-down">
      {% include "chronos/partials/week_timetable.html" with timetable=week_timetable active_day=day today=today %}
  </div>
{% endblock %}
