/*+++++++++++*/
/* Timetable */
/*+++++++++++*/
.smart-plan-badge {
    margin: 5px 20px 5px 0;
}

li.active > a > .sidenav-badge {
    background-color: whitesmoke !important;
    color: #DA3D56 !important;
}

.timetable-plan .row, .timetable-plan .col {
    display: flex;
    padding: 0 .25rem;
}

.timetable-plan .row {
    margin-bottom: .25rem;
}

.lesson-card, .timetable-title-card {
    margin: 0;
    display: flex;
    flex-grow: 1;
    min-height: 65px;
}

.supervision-card {
    min-height: 35px;
}

.lesson-card .card-content {
    padding: 0;
    text-align: center;
    width: 100%;
    height: 100%;
    display: flex;
    flex-direction: column;
}

.lesson-card .card-content > div {
    padding: 3px;
    flex: auto;
    width: 100%;
    display: flex;
    align-items: center;
    justify-content: center;
}

.timetable-title-card .card-content {
    padding: 10px;
    text-align: center;
    width: 100%;
}

.timetable-mobile-title-card {
    margin-top: 50px;
    margin-bottom: .60rem;
    display: flex;
    flex-grow: 1;
}

.timetable-mobile-title-card:first-child {
    margin-top: -10px;
    margin-bottom: .60rem;
}

.timetable-mobile-title-card .card-content {
    padding: 10px;
    text-align: center;
    width: 100%;
}

.timetable-mobile-title-card .card-content .card-title {
    font-weight: bold;
}

table.substitutions td, table.substitutions th {
    padding: 10px 5px;
}

.lesson-with-sub {
    border: 3px solid red;
    border-radius: 3px;
}

.lesson-with-sub .badge {
    margin: 0;
}

.lesson-with-event {
    border: 3px solid #9c27b0;
    border-radius: 3px;
}

.lesson-card a, .substitutions a {
    color: inherit;
}

.btn-timetable-quicklaunch {
    margin: 1%;
    width: 30%;
}


.print-body table.substitutions td, .print-body table.substitutions th {
    padding: 0;
}

.print-body span.badge.new {
    font-size: 0.9rem;
    line-height: 20px;
    height: 20px;
    margin: 2px;
    letter-spacing: 0.3pt;
}

.black-text-a a {
    color: black;
}

.holiday-badge {
    float: left !important;
    position: relative;
    margin-left: 0 !important;
    left: 50%;
    transform: translate(-50%);
    width: auto;
    height: auto !important;
    min-height: 26px;
    margin-top: 5px;
}

.holiday-card .card-content{
    width: 100%;
}

.holiday-card .holiday-badge {
    margin-top: 0;
}

@media print {
    header {
        width: 200mm;
        top: 5mm;
    }

    .header-space {
        height: 30mm;
    }

    .sheet {
        padding: 5mm;
    }

    .print-layout-table, .print-layout-table > td {
        width: 200mm;
        max-width: 200mm;
        min-width: 200mm;
    }
}
