{% if holiday %}
  <div class="row">
    <div class="col s12">
      <div class="card col s12 holiday-card">
        <div class="card-content">
          <p>
            {% include "chronos/partials/holiday.html" with holiday=holiday %}<br/>
          </p>
        </div>
      </div>
    </div>
  </div>
{% else %}
  {% for row in timetable %}
    <div class="row">
      <div class="col s4">
        {% if row.type == "period" %}
          {% include "chronos/partials/period_time.html" with period=row.period periods=periods %}
        {% endif %}
      </div>
      <div class="col s8">
        {% if row.type == "period" %}
          {% include "chronos/partials/elements.html" with elements=row.col %}
        {% else %}
          {% include "chronos/partials/supervision.html" with supervision=row.col %}
        {% endif %}
      </div>
    </div>
  {% endfor %}
{% endif %}
