# Generated by Django 3.2.13 on 2022-06-04 14:06

import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('chronos', '0010_remove_subject_unique_name_per_site'),
    ]

    operations = [
        migrations.AddField(
            model_name='extralesson',
            name='exam',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='extra_lessons', to='chronos.exam', verbose_name='Related exam'),
        ),
        migrations.AlterField(
            model_name='exam',
            name='comment',
            field=models.TextField(blank=True, default='', verbose_name='Comment'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='exam',
            name='date',
            field=models.DateField(default=datetime.date(1999, 1, 1), verbose_name='Date of exam'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='exam',
            name='period_from',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='chronos.timeperiod', verbose_name='Start period'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='exam',
            name='period_to',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='chronos.timeperiod', verbose_name='End period'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='exam',
            name='title',
            field=models.CharField(blank=True, max_length=255, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='timeperiod',
            name='weekday',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Monday'), (1, 'Tuesday'), (2, 'Wednesday'), (3, 'Thursday'), (4, 'Friday'), (5, 'Saturday'), (6, 'Sunday')], verbose_name='Week day'),
        ),
    ]
