<div class="card lesson-card {% if active_day and week_day == active_day %} z-depth-5 active {% endif %}">
  <div class="card-content">
    {% for element in elements %}
      {% if element.label_ == "lesson_period" %}
        {% include "chronos/partials/lesson.html" with lesson_period=element %}
      {% elif element.label_ == "extra_lesson" and smart %}
        {% include "chronos/partials/extra_lesson.html" with extra_lesson=element %}
      {% elif element.label_ == "event" and smart %}
        {% include "chronos/partials/event.html" with event=element %}
      {% endif %}
    {% endfor %}
  </div>
</div>
